/**
 * Responsive features
 * for WebGL.
 * Resize the canvas.
 * Respond to mouse,
 * touch, and click
 * events.
 * @param {Controller}
 * controller
 * Reference to Controller
 * @param {Function} fClick.
 */
var Responsive = function(
 controller,
 fClick
){
	
controller.responsive = this;

this.bb = null;

this.mXOffset = this.mYOffset = Number(0);

// Leave 5% between elements.
this.nGraphic = Number(0.40);

this.nText = Number(0.55);

// Click canvas function:
this.fClick = fClick;

this.getElements(
 controller
);

this.setListeners(
 controller
);
 
this.listenerResize(
 controller
);

};

Responsive.prototype = {
		
/**
 * Save needed properties
 * from the Web page
 * and WebGLProgram.
 * 
 * @param {Controller} 
 * controller
 */	
getElements: function(controller){
	
var gl = controller.gl;

var program = controller.program;
	
this.eText = document.getElementById(
'eText'
);

this.eGraphic = document.getElementById(
'eGraphic'
);

this.eContent = document.getElementById(
'eContent'
); 

this.eDebug = document.getElementById(
'eDebug'
);

this.uHeight = gl.getUniformLocation(
program, 
"u_height"
);

this.uWidth = gl.getUniformLocation(
program, 
"u_width"
);

this.uCenter = gl.getUniformLocation(
program, 
"u_center"
);
 	
},
		
/**
 * Assign event listeners:
 * onclick,mousemove,
 * touchmove, scroll,
 * and resize.
 * 
 * @param {Controller} 
 * controller
 */	
setListeners: function(controller){
	
var cv = controller.cv; 

if (this.fClick != null){
 cv.addEventListener(
 'click',
 this.clickCanvas,
 false
 );
} 
 
if ("ontouchstart" in window){

cv.addEventListener
(
 'touchmove',    
 this.listenerMoveTouch,
 false   
);       
}
	
else {
cv.addEventListener
(
 'mousemove', 
 this.listenerMoveMouse, 
 false
);          
}  
	 
window.controller = controller;
	 
window.addEventListener
(
 'scroll',  
 this.listenerScroll 
);

window.addEventListener (
 'resize', 
 this.listenerResize  
);
  
this.listenerResize(
 controller
);
},

/**
 * Respond to scroll events.
 * @param {EventObject} ev
 */
listenerScroll: function(ev){
	
var controller = ev;

var responsive;

if (ev.currentTarget != null){
 ev.preventDefault();  
 controller = ev.currentTarget.controller;
}

responsive = controller.responsive;

responsive.setOffsets(
 controller
);

},

/**
 * Respond to resize events.
 * @param {EventObject} ev
 */
listenerResize: function(ev){
	
var controller = ev;

if (ev.currentTarget != null){
	
 ev.preventDefault(); 

 controller = ev.currentTarget.controller;
}

var gl = controller.gl;
var glDemo = controller.glDemo;
var responsive = controller.responsive;
  
responsive.setElementDimensions(
 controller
);

glDemo.render(
 controller
);

if (
 responsive.uHeight != null 
 && responsive.uWidth != null)
{ 

var cv = controller.cv;
 
gl.uniform1f(
 responsive.uHeight, 
 cv.height
);

gl.uniform1f(
 responsive.uWidth, 
 cv.width
);

}

if(responsive.uCenter != null){

gl.uniform1f(
 responsive.uCenter, 
 glDemo.nMiddle
 );
} 
},

/*
 * Assign canvas area
 * and data area
 * dimensions based
 * on orientation
 * and screen resolution.
 * 
 * @param {Controller}
 * controller
 */
setElementDimensions: function(
 controller
)
{ 

var responsive = controller.responsive;
 
var gl = controller.gl;

var nW = window.innerWidth * 0.9; 
 
var nH = window.innerHeight; 
 
var nCanvasDim = Number(0);
 
var nOutputDim = Number(0);

// Landscape orientation.
if (nW > nH){
   
// For narrow phones and mobile devices
// in landscape orientation.
if (nW < 640){
	
 nCanvasDim = Math.floor(nW * 0.9);
 nOutputDim = nCanvasDim;

}

else {
	
 nCanvasDim = Math.floor(nW * responsive.nGraphic);
 nOutputDim = Math.floor(nW * responsive.nText);

}
  
}

// Portrait orientation.
else {  
// 0.9 equals content area for
// all devices and orientations.
 nCanvasDim = Math.floor(nW * 0.9);
 
 nOutputDim = nCanvasDim;
 
}

responsive.setWidths(
 controller,
 nCanvasDim, 
 nOutputDim
);

responsive.setOffsets(
 controller
); 

gl.viewport(
 0, 
 0, 
 nCanvasDim, 
 nCanvasDim
);
},

/**
 * Determine the
 * X and Y coordinates
 * of the canvas, relative
 * to the browser window.
 * 
 * @param {Controller} 
 * controller.
 */
setOffsets: function(controller){
	
 var nXScroll = Number(0);
 
 var nYScroll = Number(0);
 
 var responsive = controller.responsive;
 
 var cv = controller.cv;
 
 responsive.mXOffset = responsive.mYOffset = Number(0);
 
 var bb = cv.getBoundingClientRect();
 
 if (bb != null){    

  if (window.pageXOffset != null){
   nXScroll = window.pageXOffset;       
  }
  
  if (window.pageYOffset != null){
    nYScroll = window.pageYOffset;       
  }  

  if (bb.left > 0){
  	responsive.mXOffset = bb.left + nXScroll;
  }

  else if (nXScroll > 0){
  	responsive.mXOffset = nXScroll + bb.left;
  } 
  
  if (bb.top > 0){
  	responsive.mYOffset = bb.top + nYScroll;
  }
 
  else if (nYScroll > 0){
   	responsive.mYOffset = nYScroll + bb.top;
  } 
  
  responsive.bb = bb;

 }  
},

/**
 * @param {Controller}
 * controller.
 * @param {Number} nCD
 * Number for canvas dimensions.
 * @param{Number} nDD
 * Number for data area dimensions.
 */
setWidths: function(
 controller,
 nCD,
 nDD
)
{   
var responsive = controller.responsive;

var glDemo = controller.glDemo;

var cv = controller.cv;

glDemo.nMiddle = nCD/2;

glDemo.nDiv = Math.PI*2/nCD;

glDemo.nY = glDemo.nMiddle;

glDemo.nX = glDemo.nMiddle;

var sCD = nCD.toString()+"px";

cv.width = nCD;

cv.height = nCD; 

cv.style.width = sCD;

cv.style.height = sCD;

responsive.eGraphic.style.width = sCD;

responsive.eText.style.width = nDD.toString()+"px"; 

},

/**
 * Respond to touch
 * movement across
 * the canvas.
 * Assigns 
 * glDemo.nX,nY
 * properties:
 * Current X,Y
 * coordinates 
 * relative to the canvas.
 * 
 * @param {EventObject} ev
 */
listenerMoveTouch: function(ev){ 

ev.preventDefault();
 
var controller = ev.currentTarget.controller;
var responsive = controller.responsive;
var glDemo = controller.glDemo;
 
if (ev.targetTouches.length >=  1) {
 
var t1 = ev.targetTouches[0];  
  
glDemo.nX = t1.pageX - responsive.mXOffset; 
glDemo.nY = t1.pageY - responsive.mYOffset; 
  
glDemo.render(
 controller
);
  
}

},

/**
 * Respond to mouse
 * movement across
 * the canvas.
 * 
 * Assigns
 * glDemo.nX,nY
 * properties:
 * Current X,Y
 * coordinates 
 * relative to the canvas.
 * 
 * @param {EventObject} ev 
 */
listenerMoveMouse: function(ev){
	
var controller = ev.currentTarget.controller;

ev.preventDefault();  

var glDemo = controller.glDemo;

var responsive = controller.responsive;
 
glDemo.nX = ev.clientX - responsive.bb.left;

glDemo.nY = ev.clientY - responsive.bb.top;

glDemo.render(
 controller
);

},

/**
 * Respond to click
 * events on the canvas.
 * 
 * @param {EventObject} ev 
 */
clickCanvas: function(ev){
	
var controller = ev.currentTarget.controller;

var responsive = controller.responsive;

var bb = controller.cv.getBoundingClientRect();

var nX = Math.floor(ev.clientX - bb.left);

var nY = Math.floor(ev.clientY - bb.top);

// Activate reference
// to function from
// current project.
responsive.fClick(
 nX,
 bb.height-nY,
 controller
);

},
};