/**
 * WebGL Sprite.
 * @param {String} s 
 * path to an image.
 * @param {Number} i index with
 * multiple WebGL uses.
 * @param  {Number} nOffset
 * Offset into the sprite's buffer.
 * @param {Number} nCount
 * Number of elements from
 * the buffer.
 */
var Sprite = function(s,i,nOffset,nCount){
 
 this.sSrc = s;

 this.texture = null;

 this.uSampler = null; 

 this.aTexCoord = null;   

 this.render = null;  

 this.idx = parseInt(i);

 this.nOffset = nOffset;

 this.nCount = nCount;

 this.matrix = (
 [
  1, 0, 0, 0,
  0, 1, 0, 0,
  0, 0, 1, 0,
  0, 0, -4, 1
  ] 
 );

 return this;

}


Sprite.prototype = { 

/**
 * Obtain variables needed
 * for the sprite's texture. 
 * @param {Controller} controller
 * @returns {Number}
 * One if this sprite loads an Image.
 * Zero if this sprite doesn't load 
 * an Image.
 */
getTextureVariables: function(controller){
 
 var gl = controller.gl;
 
 var program = controller.program; 
 
 this.uSampler = gl.getUniformLocation
 (
  program, 
  "u_sampler"+this.idx
 );

 this.aTexCoord = gl.getAttribLocation
 (
  program, 
  "a_tex_coord"+this.idx
 );

 if (
  this.aTexCoord != null && 
  this.aTexCoord >= 0
 )
 {   
 
  gl.vertexAttribPointer
  (
   this.aTexCoord, 
   2, 
   gl.FLOAT, 
   gl.FALSE, 
   20, 
   12
  );   
  
  gl.enableVertexAttribArray
  (
   this.aTexCoord
  );     
 }

 if(this.sSrc != null){	

  var img = new Image();
  
  img.controller = controller;
  
  img.sprite = this;    
  
  img.onload = this.setImage;
  
  img.src =  this.sSrc;
  
  return 1;
}

return 0;

},

/**
 * Assign Image
 * data to a WebGLTexture.
 * @param {EventObject} ev.
 */ 
setImage: function(
 ev
)
{ 
 var controller = ev.currentTarget.controller;

 var sprite = ev.currentTarget.sprite;
 
 var img = ev.currentTarget;
 
 var gl = controller.gl;
 
 controller.nImagesLoaded++;
 
 gl.uniform1i
 (
	 sprite.uSampler, 
	 sprite.idx
 ); 
 
 sprite.texture = gl.createTexture();

 gl.activeTexture
 (
  gl.TEXTURE0 + sprite.idx
 ); 
  
 gl.bindTexture
 (
  gl.TEXTURE_2D, 
  sprite.texture
 );  
 
 gl.pixelStorei
 (
  gl.UNPACK_FLIP_Y_WEBGL, 
  true
 );   

 try{
   gl.texImage2D(
   gl.TEXTURE_2D,
   0, 
   gl.RGBA, 
   gl.RGBA,        
   gl.UNSIGNED_BYTE, 
   img
  );  
 }  

 catch(err){
  controller.viewGetWebGL(
   err
  );
  return;
 }  
 
 sprite.setWrapToEdges(
  gl
 );

 sprite.setMinMagFilters(
  gl
 );

if (controller.nImagesToLoad == controller.nImagesLoaded){   

 controller.setProgramVariables(
  controller
 );  
} 
},

/**
 * Wrap a texture edge to edge.
 * @param {WebGLRenderingContext} gl
 */
setWrapToEdges: function(gl){

 gl.texParameteri
 (
  gl.TEXTURE_2D,
  gl.TEXTURE_WRAP_S,
  gl.CLAMP_TO_EDGE
 );

 gl.texParameteri
 (
  gl.TEXTURE_2D,
  gl.TEXTURE_WRAP_T,
  gl.CLAMP_TO_EDGE
 );

},

/**
 * Assign NEAREST
 * minification and
 * magnification filters.
 * @param {WebGLRenderingContext} gl
 */
setMinMagFilters: function(gl){ 
 
 gl.texParameteri
 (
  gl.TEXTURE_2D, 
  gl.TEXTURE_MIN_FILTER, 
  gl.NEAREST
 );  

 gl.texParameteri
 (
  gl.TEXTURE_2D, 
  gl.TEXTURE_MAG_FILTER, 
  gl.NEAREST
 ); 
},

};